import asyncio
import concurrent.futures
import requests
from greent.util import LoggingUtil

logger = LoggingUtil.init_logging (__name__)

class AsyncUtil:
    
    @staticmethod
    async def parallel_requests (urls, process_response, degree=20):
        with concurrent.futures.ThreadPoolExecutor(max_workers=degree) as executor:
            loop = asyncio.get_event_loop()
            futures = [
                loop.run_in_executor(
                    executor, 
                    requests.get, 
                    url
                )
                for url in urls
            ]
            for response in await asyncio.gather(*futures):
                process_response (response)

    @staticmethod
    def execute_parallel_requests (urls, response_processor, chunk_size=20):
        chunk_size = 20
        chunks = [urls[i:i + chunk_size] for i in range(0, len(urls), chunk_size)]
        for chunk in chunks:
            logger.debug (" requests: {0}".format (len(chunk)))
            loop = asyncio.get_event_loop()
            loop.run_until_complete(AsyncUtil.parallel_requests(chunk, process_response=response_processor))



import json
import graphene
from graphene import resolve_only_args
from datetime import datetime, timedelta
from dateutil.parser import parse as parse_date
from greent.core import GreenT
from greent.translator import Translation

# http://graphql.org/learn/introspection/

'''
class ExposureInterface (graphene.Interface):
    start_time    = graphene.String ()
    end_time      = graphene.String ()
    exposure_type = graphene.String ()
    latitude      = graphene.String ()
    longitude     = graphene.String ()
    units         = graphene.String ()
    value         = graphene.String ()
'''
class ExposureInterface (graphene.Interface):
    date_time  = graphene.String ()
    latitude   = graphene.String ()
    longitude  = graphene.String ()
    value      = graphene.String ()

class ExposureScore (graphene.ObjectType):
    class Meta:
        interfaces = (ExposureInterface, )

class ExposureValue (graphene.ObjectType):
    class Meta:
        interfaces = (ExposureInterface, )

class ExposureCondition (graphene.ObjectType):
    chemical = graphene.String ()
    gene     = graphene.String ()
    pathway  = graphene.String ()
    pathName = graphene.String ()
    pathID   = graphene.String ()
    human    = graphene.String ()

class Drug(graphene.ObjectType):
    generic_name = graphene.String ()
          
class GenePath(graphene.ObjectType):
    uniprot_gene = graphene.String ()
    kegg_path    = graphene.String ()
    path_name    = graphene.String ()
    human        = graphene.String ()

# No map type in GraphQL: https://github.com/facebook/graphql/issues/101

class PatientVisit(graphene.ObjectType):
    date = graphene.String ()
    visit_type = graphene.String ()

class Location(graphene.ObjectType):
    latitude = graphene.String ()
    longitude = graphene.String ()

class Prescription(graphene.ObjectType):
    medication = graphene.String ()
    date       = graphene.String ()

class Diagnosis(graphene.ObjectType):
    diagnosis  = graphene.String ()
    visit      = graphene.Field (PatientVisit)
    
class Patient(graphene.ObjectType):
    birth_date    = graphene.String ()
    race          = graphene.String ()
    sex           = graphene.String ()
    patient_id    = graphene.String ()
    geo_code      = graphene.Field (Location)
    prescriptions = graphene.List (Prescription)
    diagnoses     = graphene.List (Diagnosis)

class DrugToDisease (graphene.ObjectType):
    drug_name = graphene.String ()
    target_name = graphene.String ()
    disease_name = graphene.String ()

class Attribute(graphene.ObjectType):
    key = graphene.String ()
    value = graphene.String ()
    
class Thing(graphene.ObjectType):
    value      = graphene.String ()
    attributes = graphene.List (of_type=Attribute)
    
greenT = GreenT (config='greent.conf', override={
    "clinical_url" : "http://localhost:5000/patients"
})

class GreenQuery (graphene.ObjectType):

    endotype = graphene.List(of_type=graphene.String,
                             query=graphene.String ())

    exposure_score = graphene.List (of_type=ExposureScore,
                                    exposureType  = graphene.String (),
                                    startDate     = graphene.String (),
                                    endDate       = graphene.String (),
                                    exposurePoint = graphene.String ())
    
    exposure_value = graphene.List (of_type=ExposureValue,
                                    exposureType  = graphene.String (),
                                    startDate     = graphene.String (),
                                    endDate       = graphene.String (),
                                    exposurePoint = graphene.String ())

    patients = graphene.List (of_type=Patient,
                              age=graphene.Int (),
                              race=graphene.String (),
                              sex=graphene.String ())
    
    exposure_conditions = graphene.List (of_type=ExposureCondition,
                                         chemicals = graphene.List(graphene.String))
    
    drugs_by_condition = graphene.List(of_type=Drug,
                                       conditions = graphene.List(graphene.String))
    
    gene_paths_by_disease = graphene.List (of_type=GenePath,
                                           diseases = graphene.List(graphene.String))

    drug_gene_disease = graphene.List (of_type=DrugToDisease,
                                       drug_name = graphene.String (),
                                       disease_name = graphene.String ())
    
    translate = graphene.List (of_type = Thing,
                               thing   = graphene.String (),
                               domainA = graphene.String (),
                               domainB = graphene.String ())

    '''
class Attribute(graphene.ObjectType):
    key = graphene.String ()
    value = graphene.String ()
    
class Thing(graphene.ObjectType):
    value      = graphene.String ()
    attributes = graphene.List (of_type=Attribute)

    '''
    def resolve_endotype (obj, args, context, info):
        return greenT.endotype.get_endotype (json.loads (args.get("query")))
    
    def resolve_translate (obj, args, context, info):
        translation = Translation (obj=args.get("thing"),
                                   type_a=args.get("domainA"),
                                   type_b=args.get("domainB"))
        return list (map (lambda v : Thing (value=v), greenT.translator.translate (translation)))
    
    def resolve_exposure_score (obj, args, context, info):
        result = None
        result = greenT.get_exposure_scores (
            exposure_type  = args.get ("exposureType"),
            start_date     = args.get ("startDate"),
            end_date       = args.get ("endDate"),
            exposure_point = args.get ("exposurePoint"))
        out = []
        for r in result['scores']:
            latitude, longitude = r['latLon'].split (",")
            out.append (ExposureValue (date_time  = datetime.strftime (r['dateTime'], "%Y-%m-%d"),
                                       latitude   = latitude,
                                       longitude  = longitude,
                                       value      = r['score']))
        return out

    def resolve_exposure_value (obj, args, context, info):
        result = None
        result = greenT.get_exposure_values (
            exposure_type  = args.get ("exposureType"),
            start_date     = args.get ("startDate"),
            end_date       = args.get ("endDate"),
            exposure_point = args.get ("exposurePoint"))
        out = []
        for r in result['values']:
            latitude, longitude = r['latLon'].split (",")
            out.append (ExposureValue (date_time  = datetime.strftime (r['dateTime'], "%Y-%m-%d"),
                                       latitude   = latitude,
                                       longitude  = longitude,
                                       value      = r['value']))
        return out

    def resolve_patients (obj, args, context, info):
        result = None
        
        result = greenT.get_patients (
            age = args.get ("age"),
            sex = args.get ("sex"),
            race = args.get ("race"),
            location = args.get ("location"))

        out = []
        for r in result:
            diagnoses = []
            for key, value in r['diag'].items ():
                visit_date = list (value)[0]
                visit_type = value[visit_date]
                diagnosis = list (list (value)[0])[1]
                diagnoses.append (Diagnosis (
                    diagnosis = key,
                    visit = PatientVisit (date = visit_date,
                                          visit_type = visit_type)))

            prescriptions = []
            for key, value in r['medList'].items ():
                prescriptions.append (Prescription (
                    medication = key,
                    date       = value))
                
            out.append (Patient (
                birth_date    = r['birth_date'],
                race          = r['race'],
                sex           = r['sex'],
                patient_id    = r['patient_id'],
                geo_code      = Location (r['geoCode']['GEO:LAT'], r['geoCode']['GEO:LONG']),
                prescriptions = prescriptions,
                diagnoses     = diagnoses))

        return out

    def resolve_exposure_conditions (obj, args, context, info):
        obj = args.get ("chemicals")
        result = greenT.get_exposure_conditions (chemicals = obj)
        if result:
            out = []
            for r in result:
                out.append (ExposureCondition (
                    chemical = r["chemical"],
                    gene     = r["gene"],
                    pathway  = r["pathway"],
                    pathName = r["pathName"],
                    pathID   = r["pathID"],
                    human    = r["human"] ))
                result = out
        return result

    def resolve_drugs_by_condition (obj, args, context, info):
        conditions = args.get ("conditions")
        diseases = greenT.get_drugs_by_condition (conditions = conditions)
        return list(map(lambda s : Drug(s), diseases))

    def resolve_gene_paths_by_disease (obj, args, context, info):
        diseases = args.get ("diseases")
        gene_paths = greenT.get_genes_pathways_by_disease (diseases = diseases)
        return list(map(lambda g : GenePath (
            uniprot_gene = g['uniprotGene'],
            kegg_path    = g['keggPath'],
            path_name    = g['pathName'],
            human        = g['human']), gene_paths))
    def resolve_drug_gene_disease (obj, args, context, info):
        drug_name = args.get ("drug_name")
        disease_name = args.get ("disease_name")
        paths = greenT.get_drug_gene_disease (disease_name=disease_name, drug_name=drug_name)
        return list(map(lambda dd : DrugToDisease (
            drug_name = drug_name,
            target_name = dd['uniprotSymbol'],
            disease_name = disease_name), paths))

Schema = graphene.Schema(query=GreenQuery)


'''
{
  translate (thing:"Imatinib",
    domainA: "http://chem2bio2rdf.org/drugbank/resource/Generic_Name",
  	domainB: "http://chem2bio2rdf.org/uniprot/resource/gene")
  {
    type
    value
  }
}

{
  translate (thing:"DOID:2841",
    domainA: "http://identifiers.org/doid/",
  	domainB: "http://identifiers.org/mesh/disease/id")
  {
    type
    value
  }
}


{
  translate (thing:"Asthma",
					  domainA :	"http://identifiers.org/mesh/disease/name/",
						domainB : "http://identifiers.org/mesh/drug/name/")
  {
  	type
    value
  }
}


{
  exposureValue(exposureType: "pm25", 
    		startDate: "2010-01-06", 
    		endDate: "2010-01-06", 
    	        exposurePoint: "35.9131996,-79.0558445") {
    value
  }
}

{
  exposureScore(exposureType: "pm25", 
    		startDate: "2010-01-06", 
    		endDate: "2010-01-06", 
    		exposurePoint: "35.9131996,-79.0558445") {
    value
  }
}

{
  exposureConditions (chemicals: [ "D052638" ] ) {
    chemical
    gene
    pathway
    pathName
    pathID
    human
  } 
}

{
  drugsByCondition (conditions: [ "d001249" ] ) {
    genericName
  } 
}

{
  genePathsByDisease (diseases: [ "d001249" ] ) {
    uniprotGene
    keggPath
    pathName
  } 
}

{
  patients {
    birthDate
    patientId
    geoCode {
      latitude
      longitude
    }
    prescriptions {
      date
      medication
    }
    diagnoses {
      diagnosis
      visit {
        date
        visitType
      }
    }
  }
}


{
  __type(name: "Patient") {
    name
    fields {
      name
      type {
        name
        kind
      }
    }
  }
}

'''



import requests
import json

class Neo4JREST(object):
    def __init__(self, url):
        self.url = url
        self.query_endpoint = "{0}/db/data/transaction".format (self.url)
        self.schema_endpoint = "{0}/db/data/schema/".format (self.url)

    def request (self, url, obj):
        return requests.post (url = self.query_endpoint,
                             data = json.dumps (obj, indent=2),
                             headers={ "Content-Type" : "application/json" }).json ()
        
    def query (self, query, labels=None, node_properties=None):
        response = self.request (
            url = self.query_endpoint,
            obj = {
                "statements": [
                    {
                        "statement": query,
                        "resultDataContents": [
                            "row",
                            "graph"
                        ],
                        "includeStats": True
                }
                ]
            })
        if node_properties != None:
            response = self.filter_nodes (response, labels, node_properties)
        return response
    
    def filter_nodes (self, response, labels=None, properties=['identifier']):
        nodes = []
        for r in response['results']:
            for d in r['data']:
                for n in d['graph']['nodes']:
                    if labels != None:
                        if any (map (lambda b : b in n['labels'], labels)):
                            obj = {}
                            for prop in properties:
                                obj[prop] = n['properties'][prop]
                            nodes.append (obj)
                for s in d['graph']['relationships']:
                    pass
        return nodes



import os
import json
import operator
import sys
import traceback

# pyswip
from pyswip import Prolog
from pyswip import registerForeign
from pyswip import Functor
from pyswip import call
from pyswip import newModule
from pyswip import Variable
from pyswip import Query
from pyswip.easy import getList, registerForeign

# problog
from problog.program import PrologFile
from problog.program import PrologString
from problog.formula import LogicFormula
from problog import get_evaluatable
from problog.logic import Term
from problog.logic import Var
from problog.logic import Constant
from problog.engine_stack import StackBasedEngine
from problog.engine import DefaultEngine

def test_problog():

    from problog.program import PrologString
    from problog import get_evaluatable

    text = open("translator_knowledgebase.prolog", 'r').read ()
    pl_model = PrologString(text)
    engine = DefaultEngine()
    db = engine.prepare(pl_model)

    cbr_gene = Term("cbr_gene")
    doid = Term("doid")
    hgnc_id = Term("hgnc_id")
    hetio_cell = Term("hetio_cell")
    translates = Term("translates")
    path_to = Term("path_to")
    query_term = path_to (cbr_gene, hetio_cell, None)

    doid_to_hgnc = path_to (doid, hgnc_id, None)
    res = engine.query (db, doid_to_hgnc)
    print ('%s? %s' % (doid_to_hgnc, res))

    reify = Term("reify")
    drug = Term("tS")
    res = engine.query(db, reify (drug, None, None))
    print ('%s? %s' % (reify, res))

    type_matrix   = Term ("type_matrix")
    disease       = Term ("tD")
    gene          = Term ("tG")
    res           = engine.query(db, type_matrix (disease, None, None, gene, None, None))
    print ('%s? %s' % (reify, res))
    parse_terms = lambda t : list(map(lambda s : Term(s), str(t).replace("[","").replace("]","").replace(",","").split (" ")))
    for r in res:
        L = parse_terms (r[2])
        R = parse_terms (r[5])
        print (" : %s %s " % (L, R))
        for anL in L:
            for anR in R:
                path = path_to (anL, anR, None)
                res = engine.query (db, path)
                if len(res) > 0:
                    print ('   -- %s? => %s' % (path, res))

    sys.exit (0)


test_problog ()




if __name__ == "__main__":
    p = Prolog ()
    p.consult 






import logging
import unittest
import json
from pprint import pprint, pformat
from core import GreenT

class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name):
        FORMAT = '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        logging.basicConfig(format=FORMAT, level=logging.INFO)
        return logging.getLogger(name)

logger = LoggingUtil.init_logging (__file__)

class TestGreenT(unittest.TestCase):

    greenT = GreenT ()

    def test_exposure_scores (self):
        exposure_type = "pm25"
        start_date = "2010-1-1"
        end_date = "2010-1-7"
        lat = "35.9131996"
        lon = "-79.0558445"
        exposure_point = ",".join ([ lat, lon ])
        results = self.greenT.get_exposure_scores (exposure_type, start_date, end_date, exposure_point)
        for r in results['scores']:
            this_lat, this_lon = r['latLon'].split (',')
            print ("   -- lat: (out:{0}) (in:{1}) lon: (out:{2}) (in:{3}) score: {4}".format (
                this_lat, lat, this_lon, lon, r['score']))
            self.assertTrue (this_lat == lat and this_lon == lon)
        
    def test_exposure_values (self):
        exposure_type = "pm25"
        start_date = "2010-1-1"
        end_date = "2010-1-7"
        lat = "35.9131996"
        lon = "-79.0558445"
        exposure_point = ",".join ([ lat, lon ])
        results = self.greenT.get_exposure_values (exposure_type, start_date, end_date, exposure_point)
        for r in results['values']:
            this_lat, this_lon = r['latLon'].split (',')
            print ("   -- lat: (out:{0}) (in:{1}) lon: (out:{2}) (in:{3}) value: {4}".format (
                this_lat, lat, this_lon, lon, r['value']))
            self.assertTrue (this_lat == lat and this_lon == lon)

    def test_chembio (self):
        chemicals = [ 'D052638' ]
        conditions = self.greenT.get_exposure_conditions (chemicals)
        t = False
        for c in conditions:
            if c["gene"] == "http://chem2bio2rdf.org/uniprot/resource/gene/IL6":
                t = True
                break
        self.assertTrue (t)
        print (json.dumps (conditions[:2], indent=2))
        drugs = self.greenT.get_drugs_by_condition (conditions=[ "d001249" ])
        for d in [ "Paricalcitol", "NIMESULIDE", "Ramipril" ]:
            self.assertTrue (d in drugs)

    def test_clinical (self):
        '''
        104855982
        104855982
        104855982
        100493218
        '''

        '''
        result = self.greenT.clinical.get_patients (age='4', sex='male', race='white', location='OUTPATIENT')
        result = sorted (result, key = lambda e : e['patient_id'] )
        self.assertTrue ( result[0]['patient_id'] == '104855982' )

        result = self.greenT.clinical.get_patients (age='4', sex='male', race='white')
        result = sorted (result, key = lambda e : e['patient_id'] )
        self.assertTrue ( result[0]['patient_id'] == '104855982' )


        result = self.greenT.clinical.get_patients (age='4', sex='male')
        result = sorted (result, key = lambda e : e['patient_id'] )
        self.assertTrue ( result[0]['patient_id'] == '104855982' )
        
        result = self.greenT.clinical.get_patients (age='4')
        result = sorted (result, key = lambda e : e['patient_id'] )
        self.assertTrue (result[0]['patient_id'] ==  '100493218' )
        '''
        pass
    
if __name__ == '__main__':
    unittest.main()

